@@ -24,7 +24,7 @@ from mechanize import HTTPRedirectHandler, \
     HTTPCookieProcessor, HTTPRefererProcessor, \
     HTTPErrorProcessor, HTTPHandler
 from mechanize import OpenerDirector, build_opener, Request
-from mechanize._urllib2_fork import AbstractHTTPHandler, normalize_url
+from mechanize._urllib2_fork import AbstractHTTPHandler, normalize_url, AbstractBasicAuthHandler
 from mechanize._util import write_file
 
 import mechanize._response
@@ -69,6 +69,10 @@ class TrivialTests(mechanize._testcase.TestCase):
             self.assertEqual(
                 mechanize._urllib2_fork.parse_http_list(string), list)
 
+    def test_parse_authreq(self):
+        for bad in (",,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,",):
+            self.assertIsNone(AbstractBasicAuthHandler.rx.search(bad))
+
 
 def test_request_headers_dict():
     """
@@ -1867,11 +1871,11 @@ class RequestTests(unittest.TestCase):
         self.assertEqual("POST", self.get.get_method())
 
     def test_get_full_url(self):
-        self.assertEqual("http://www.python.org/%7Ejeremy/",
+        self.assertEqual("http://www.python.org/~jeremy/",
                          self.get.get_full_url())
 
     def test_selector(self):
-        self.assertEqual("/%7Ejeremy/", self.get.get_selector())
+        self.assertEqual("/~jeremy/", self.get.get_selector())
         req = Request("http://www.python.org/")
         self.assertEqual("/", req.get_selector())
 
