@@ -2,30 +2,32 @@
 
 """Functional tests from the Python standard library test suite."""
 
-import mimetools
 import threading
-import urlparse
 import mechanize
-import BaseHTTPServer
 import unittest
 
 from mechanize._testcase import TestCase
 from mechanize._urllib2_fork import md5_digest
+from mechanize.polyglot import is_py2, urlparse, HTTPMessage
 
 import testprogram
 
 
+if is_py2:
+    from BaseHTTPServer import HTTPServer, BaseHTTPRequestHandler
+else:
+    from http.server import HTTPServer, BaseHTTPRequestHandler
+
 # Loopback http server infrastructure
 
-class LoopbackHttpServer(BaseHTTPServer.HTTPServer):
+
+class LoopbackHttpServer(HTTPServer):
     """HTTP server w/ a few modifications that make it useful for
     loopback testing purposes.
     """
 
     def __init__(self, server_address, RequestHandlerClass):
-        BaseHTTPServer.HTTPServer.__init__(self,
-                                           server_address,
-                                           RequestHandlerClass)
+        HTTPServer.__init__(self, server_address, RequestHandlerClass)
 
         # Set the timeout of our listening socket really low so
         # that we can stop the server easily.
@@ -43,18 +45,19 @@ class LoopbackHttpServer(BaseHTTPServer.HTTPServer):
 
         return (request, client_address)
 
+
 class LoopbackHttpServerThread(threading.Thread):
     """Stoppable thread that runs a loopback http server."""
 
     def __init__(self, handle_request=None):
         threading.Thread.__init__(self)
-        self._stop = False
+        self._keep_going = True
         self.ready = threading.Event()
         self._request_handler = None
         if handle_request is None:
             handle_request = self._handle_request
         self.httpd = LoopbackHttpServer(('127.0.0.1', 0), handle_request)
-        #print "Serving HTTP on %s port %s" % (self.httpd.server_name,
+        # print "Serving HTTP on %s port %s" % (self.httpd.server_name,
         #                                      self.httpd.server_port)
         self.port = self.httpd.server_port
 
@@ -69,17 +72,18 @@ class LoopbackHttpServerThread(threading.Thread):
         """Stops the webserver if it's currently running."""
 
         # Set the stop flag.
-        self._stop = True
+        self._keep_going = False
 
         self.join()
 
     def run(self):
         self.ready.set()
-        while not self._stop:
+        while self._keep_going:
             self.httpd.handle_request()
 
 # Authentication infrastructure
 
+
 class DigestAuthHandler:
     """Handler for performing digest authentication."""
 
@@ -108,7 +112,7 @@ class DigestAuthHandler:
 
     def _create_auth_dict(self, auth_str):
         first_space_index = auth_str.find(" ")
-        auth_str = auth_str[first_space_index+1:]
+        auth_str = auth_str[first_space_index + 1:]
 
         parts = auth_str.split(",")
 
@@ -142,18 +146,18 @@ class DigestAuthHandler:
         return response == auth_dict["response"]
 
     def _return_auth_challenge(self, request_handler):
-        request_handler.send_response(407, "Proxy Authentication Required")
+        request_handler.send_response(407, b"Proxy Authentication Required")
         request_handler.send_header("Content-Type", "text/html")
         request_handler.send_header(
             'Proxy-Authenticate', 'Digest realm="%s", '
             'qop="%s",'
-            'nonce="%s", ' % \
+            'nonce="%s", ' %
             (self._realm_name, self._qop, self._generate_nonce()))
         # XXX: Not sure if we're supposed to add this next header or
         # not.
-        #request_handler.send_header('Connection', 'close')
+        # request_handler.send_header('Connection', 'close')
         request_handler.end_headers()
-        request_handler.wfile.write("Proxy Authentication Required.")
+        request_handler.wfile.write(b"Proxy Authentication Required.")
         return False
 
     def handle_request(self, request_handler):
@@ -168,14 +172,14 @@ class DigestAuthHandler:
         if len(self._users) == 0:
             return True
 
-        if not request_handler.headers.has_key('Proxy-Authorization'):
+        if 'Proxy-Authorization' not in request_handler.headers:
             return self._return_auth_challenge(request_handler)
         else:
             auth_dict = self._create_auth_dict(
                 request_handler.headers['Proxy-Authorization']
-                )
-            if self._users.has_key(auth_dict["username"]):
-                password = self._users[ auth_dict["username"] ]
+            )
+            if auth_dict["username"] in self._users:
+                password = self._users[auth_dict["username"]]
             else:
                 return self._return_auth_challenge(request_handler)
             if not auth_dict.get("nonce") in self._nonces:
@@ -201,7 +205,8 @@ class DigestAuthHandler:
 
 # Proxy test infrastructure
 
-class FakeProxyHandler(BaseHTTPServer.BaseHTTPRequestHandler):
+
+class FakeProxyHandler(BaseHTTPRequestHandler):
     """This is a 'fake proxy' that makes it look like the entire
     internet has gone down due to a sudden zombie invasion.  It main
     utility is in providing us with authentication support for
@@ -214,24 +219,24 @@ class FakeProxyHandler(BaseHTTPServer.BaseHTTPRequestHandler):
         # This has to be set before calling our parent's __init__(), which will
         # try to call do_GET().
         self.digest_auth_handler = digest_auth_handler
-        BaseHTTPServer.BaseHTTPRequestHandler.__init__(self, *args, **kwargs)
+        BaseHTTPRequestHandler.__init__(self, *args, **kwargs)
 
     def log_message(self, format, *args):
         # Uncomment the next line for debugging.
-        #sys.stderr.write(format % args)
+        # sys.stderr.write(format % args)
         pass
 
     def do_GET(self):
-        (scm, netloc, path, params, query, fragment) = urlparse.urlparse(
+        (scm, netloc, path, params, query, fragment) = urlparse(
             self.path, 'http')
         self.short_path = path
         if self.digest_auth_handler.handle_request(self):
             self.send_response(200, "OK")
             self.send_header("Content-Type", "text/html")
             self.end_headers()
-            self.wfile.write("You've reached %s!<BR>" % self.path)
-            self.wfile.write("Our apologies, but our server is down due to "
-                              "a sudden zombie invasion.")
+            self.wfile.write(("You've reached %s!<BR>" % self.path).encode('utf-8'))
+            self.wfile.write(b"Our apologies, but our server is down due to "
+                             b"a sudden zombie invasion.")
 
 
 def make_started_server(make_request_handler=None):
@@ -256,6 +261,7 @@ class ProxyAuthTests(TestCase):
         digest_auth_handler.set_users({self.USER: self.PASSWD})
         digest_auth_handler.set_realm(self.REALM)
         digest_auth_handler.set_qop(qop)
+
         def create_fake_proxy_handler(*args, **kwargs):
             return FakeProxyHandler(digest_auth_handler, *args, **kwargs)
         return make_started_server(create_fake_proxy_handler)
@@ -268,13 +274,14 @@ class ProxyAuthTests(TestCase):
         server = self.get_cached_fixture(fixture_name)
 
         proxy_url = "http://127.0.0.1:%d" % server.port
-        handler = mechanize.ProxyHandler({"http" : proxy_url})
+        handler = mechanize.ProxyHandler({"http": proxy_url})
         self.proxy_digest_handler = mechanize.ProxyDigestAuthHandler()
-        self.opener = mechanize.build_opener(handler, self.proxy_digest_handler)
+        self.opener = mechanize.build_opener(
+            handler, self.proxy_digest_handler)
 
     def test_proxy_with_bad_password_raises_httperror(self):
         self.proxy_digest_handler.add_password(self.REALM, self.URL,
-                                               self.USER, self.PASSWD+"bad")
+                                               self.USER, self.PASSWD + "bad")
         self.assertRaises(mechanize.HTTPError,
                           self.opener.open,
                           self.URL)
@@ -310,7 +317,7 @@ class ProxyAuthTests(TestCase):
             result.close()
 
 
-class RecordingHTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
+class RecordingHTTPRequestHandler(BaseHTTPRequestHandler):
 
     server_version = "TestHTTP/"
     protocol_version = "HTTP/1.0"
@@ -322,11 +329,13 @@ class RecordingHTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
         self._get_next_response = get_next_response
         self._record_request = record_request
         self._record_received_headers = record_received_headers
-        BaseHTTPServer.BaseHTTPRequestHandler.__init__(self, *args, **kwds)
+        BaseHTTPRequestHandler.__init__(self, *args, **kwds)
 
     def do_GET(self):
         body = self.send_head()
         if body:
+            if not isinstance(body, bytes):
+                body = body.encode('utf-8')
             self.wfile.write(body)
 
     def do_POST(self):
@@ -404,7 +413,7 @@ class TestUrlopen(TestCase):
         return handler
 
     def test_redirection(self):
-        expected_response = 'We got here...'
+        expected_response = b'We got here...'
         responses = [
             (302, [('Location', 'http://localhost:%s/somewhere_else')], ''),
             (200, [], expected_response)
@@ -416,39 +425,40 @@ class TestUrlopen(TestCase):
         data = f.read()
         f.close()
 
-        self.assertEquals(data, expected_response)
-        self.assertEquals(handler.requests, ['/', '/somewhere_else'])
+        self.assertEqual(data, expected_response)
+        self.assertEqual(handler.requests, ['/', '/somewhere_else'])
 
     def test_404(self):
-        expected_response = 'Bad bad bad...'
+        expected_response = b'Bad bad bad...'
         handler = self._make_request_handler([(404, [], expected_response)])
+        f = None
 
         try:
             mechanize.urlopen('http://localhost:%s/weeble' % handler.port)
-        except mechanize.URLError, f:
-            pass
+        except mechanize.URLError as e:
+            f = e
         else:
             self.fail('404 should raise URLError')
 
         data = f.read()
         f.close()
 
-        self.assertEquals(data, expected_response)
-        self.assertEquals(handler.requests, ['/weeble'])
+        self.assertEqual(data, expected_response)
+        self.assertEqual(handler.requests, ['/weeble'])
 
     def test_200(self):
-        expected_response = 'pycon 2008...'
+        expected_response = b'pycon 2008...'
         handler = self._make_request_handler([(200, [], expected_response)])
 
         f = mechanize.urlopen('http://localhost:%s/bizarre' % handler.port)
         data = f.read()
         f.close()
 
-        self.assertEquals(data, expected_response)
-        self.assertEquals(handler.requests, ['/bizarre'])
+        self.assertEqual(data, expected_response)
+        self.assertEqual(handler.requests, ['/bizarre'])
 
     def test_200_with_parameters(self):
-        expected_response = 'pycon 2008...'
+        expected_response = b'pycon 2008...'
         handler = self._make_request_handler([(200, [], expected_response)])
 
         f = mechanize.urlopen('http://localhost:%s/bizarre' % handler.port,
@@ -456,42 +466,42 @@ class TestUrlopen(TestCase):
         data = f.read()
         f.close()
 
-        self.assertEquals(data, expected_response)
-        self.assertEquals(handler.requests, ['/bizarre', 'get=with_feeling'])
+        self.assertEqual(data, expected_response)
+        self.assertEqual(handler.requests, ['/bizarre', b'get=with_feeling'])
 
     def test_sending_headers(self):
-        handler = self._make_request_handler([(200, [], "we don't care")])
+        handler = self._make_request_handler([(200, [], b"we don't care")])
 
         req = mechanize.Request("http://localhost:%s/" % handler.port,
-                              headers={'Range': 'bytes=20-39'})
+                                headers={'Range': 'bytes=20-39'})
         mechanize.urlopen(req)
         self.assertEqual(handler.received_headers['Range'], 'bytes=20-39')
 
     def test_basic(self):
-        handler = self._make_request_handler([(200, [], "we don't care")])
+        handler = self._make_request_handler([(200, [], b"we don't care")])
 
         open_url = mechanize.urlopen("http://localhost:%s" % handler.port)
         for attr in ("read", "close", "info", "geturl"):
             self.assertTrue(hasattr(open_url, attr), "object returned from "
-                         "urlopen lacks the %s attribute" % attr)
+                            "urlopen lacks the %s attribute" % attr)
         try:
             self.assertTrue(open_url.read(), "calling 'read' failed")
         finally:
             open_url.close()
 
     def test_info(self):
-        handler = self._make_request_handler([(200, [], "we don't care")])
+        handler = self._make_request_handler([(200, [], b"we don't care")])
 
         open_url = mechanize.urlopen("http://localhost:%s" % handler.port)
         info_obj = open_url.info()
-        self.assertTrue(isinstance(info_obj, mimetools.Message),
-                     "object returned by 'info' is not an instance of "
-                     "mimetools.Message")
+        self.assertTrue(isinstance(info_obj, HTTPMessage),
+                        "object returned by 'info' is not an instance of "
+                        "HTTPMessage")
         self.assertEqual(info_obj.getsubtype(), "plain")
 
     def test_geturl(self):
         # Make sure same URL as opened is returned by geturl.
-        handler = self._make_request_handler([(200, [], "we don't care")])
+        handler = self._make_request_handler([(200, [], b"we don't care")])
 
         open_url = mechanize.urlopen("http://localhost:%s" % handler.port)
         url = open_url.geturl()
